<?php

/* vim: set expandtab tabstop=4 shiftwidth=4 foldmethod=marker: */
// $HeadURL$
// +------------------------------------------------------------------------+
// | Maintain - DNS/DHCP Managment Tool                                     |
// +------------------------------------------------------------------------+
// | Copyright (c) 2004 Open Source University                              |
// +------------------------------------------------------------------------+
// | This program is free software; you can redistribute it and/or          |
// | modify it under the terms of the GNU General Public License            |
// | as published by the Free Software Foundation; either version 2         |
// | of the License, or (at your option) any later version.                 |
// |                                                                        |
// | This program is distributed in the hope that it will be useful,        |
// | but WITHOUT ANY WARRANTY; without even the implied warranty of         |
// | MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the          |
// | GNU General Public License for more details.                           |
// |                                                                        |
// | You should have received a copy of the GNU General Public License      |
// | along with this program; if not, write to the Free Software            |
// | Foundation, Inc., 59 Temple Place - Suite 330,                         |
// | Boston, MA  02111-1307, USA.                                           |
// +------------------------------------------------------------------------+
// | Author(s): Zack Bartel <bartel@osuosl.org                              |
// |            Frederic Wenzel <fred@osuosl.org>                           |
// +------------------------------------------------------------------------+
// $Id: RangeMergeForm.php 3964 2006-01-05 21:49:51Z bartel $

/**
 *  RangeMergeForm.php - contains the {@link RangeMergeForm} class
 *
 *  @author $Author$
 *  @package    htdocs
 *  @subpackage forms
 *  @filesource
 */

require_once(FORMDIR."/MaintainForm.php");

class RangeMergeForm extends MaintainForm
{

    var $_range;
//{{{   form_init_elements()

    /**
     *  This method initializes the form elements that will be used.  Please refer
     *  to {@link RangeMergeForm} for more information.
     */
    function form_init_elements()
    {
        
        $frs = new FEText(_("Source Range"), false);
        $frs->set_readonly(true);
        
        $fre = new FEListBox(_("Merge Range"), false);

        $fre->set_list_data($this->get_mergeable_ranges());

       
        $this->add_elements($frs, $fre);
    }

//}}}
//{{{   form_init_data()
   
    /**
     *  This method sets the initial values for the form elements based off of
     *  which MX record we're updating.
     */
    function form_init_data()
    {
        $range = $this->getRange();
        $this->set_element_value(_("Source Range"), $range->getDisplay());
    }

//}}}
//{{{   form_hidden()

    /**
     *  This method establishes the hidden elements for the form.
     *
     *  @return Container   Container with the hidden elements for the form
     */
    function form_hidden()
    {
        $action =  new FEHidden(_("action"), $_REQUEST['action']);
        $id = new FEHidden(_("id"), $_REQUEST['id']);

        $this->add_elements($action, $id);

        return container($this->element_form(_('action')), $this->element_form(_('id')));                                
    }

//}}}
//{{{   form_action()

    /**
     *  This this function gets the two ranges to be merged and attempts
     *  to mergr them with the Range::merge_ranges() function.
     *
     *  @return Boolean True if action succeeds
     */
    function form_action()
    {
        global $user;
        
        $res = true;

        switch(strtolower($this->get_action())) {

        case 'submit':
            $vals = $this->get_all_element_values();
        
            $range1 = $_REQUEST['id'];
            $range2 = $vals['Merge_Range'];
            $cool = Range::merge_ranges($range1, $range2);

            if (PEAR::isError($cool)) {
                $this->add_error(_("ERROR"), _("Can not merge slected ranges"));
                $res = false;
            }

            break;
        default:
            die_hard(_("Invalid form action"));
            break;

        }

        return $res;
    }

//}}}
//{{{   form_success()

    /**
     *  This displays a happy message to the user letting them know that the update
     *  worked.
     *
     *  @return DIVTag  $div    Div that contains the success message
     */
    function form_success() 
    {
        $div = html_div('rsuccess');
        $div->add(_("Successfully Merged Ranges"), html_br());
        $home = html_a("index.php", _("[Return to Home]"));
        $r = html_a("range.php", _("[Return to Range List]"));
        $div->add(container($home, $r));
        //$this->getRange();
        //header("Location: ".$_SERVER['SCRIPT_NAME']."?action=edit&id=".$_REQUEST['id']);
        return $div;
    }

//}}}

//{{{ form_backend_validation()
    /**
     *  Validate selected ip addresses to ensure valid range. Return true or false depending on
     *  validity (who would have thunk it!)
     *
     */
    function form_backend_validation()
    {
        $result = true;
        return $result;
    }
//}}}
  
//{{{   get_mergeable_ranges()
    /**
     *  This function returns an array range_id=>string representations of
     *  ranges that can be merged with this one.
     *
     *  
     *  @return array $ranges
     */
    function get_mergeable_ranges()
    {
        $range = $this->getRange();
        $result = array();
        
        /* Get the ranges that can be merged with this one */
        $ranges = $range->get_mergeable_ranges();
        
        /* Put the mergeable ranges into a pk=>value array */
        foreach($ranges as $range_id)  {
            $r = new Range($range_id);
            $result[$r->getDisplay()] = $r->id;
        }
        return $result;
    }
//}}}
//{{{   getRange()
    /**
     *  This function sets the value ot $this->_range to the Range object
     *  with the pk of $_REQUEST['id']
     *
     *
     *  @return array $this->_range
     */

    function getRange()
    {
        if (!$this->_range) {
            $this->_range = new Range($_REQUEST['id']);
        }
        return $this->_range;
    }
//}}}
}

?>
