<?php

/* vim: set expandtab tabstop=4 shiftwidth=4 foldmethod=marker: */
// $HeadURL$
// +------------------------------------------------------------------------+
// | Maintain - DNS/DHCP Managment Tool                                     |
// +------------------------------------------------------------------------+
// | Copyright (c) 2004 Open Source University                              |
// +------------------------------------------------------------------------+
// | This program is free software; you can redistribute it and/or          |
// | modify it under the terms of the GNU General Public License            |
// | as published by the Free Software Foundation; either version 2         |
// | of the License, or (at your option) any later version.                 |
// |                                                                        |
// | This program is distributed in the hope that it will be useful,        |
// | but WITHOUT ANY WARRANTY; without even the implied warranty of         |
// | MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the          |
// | GNU General Public License for more details.                           |
// |                                                                        |
// | You should have received a copy of the GNU General Public License      |
// | along with this program; if not, write to the Free Software            |
// | Foundation, Inc., 59 Temple Place - Suite 330,                         |
// | Boston, MA  02111-1307, USA.                                           |
// +------------------------------------------------------------------------+
// | Author(s): Zack Bartel  <bartel@osuosl.org>                            |
// |            Michael Clay <claym@osuosl.org>                             |
// |            Frederic Wenzel <fred@osuosl.org>                           |
// +------------------------------------------------------------------------+
// $Id: SearchForm.php 4173 2006-02-08 21:31:09Z bartel $

/**
 *  SearchForm.php - contains the {@link SearchForm} class
 *
 *  @author $Author$
 *  @package    htdocs
 *  @subpackage forms
 *  @filesource
 */

class SearchForm extends FormContent
{

//{{{   vars
    var $object_data = array();
//}}}
//{{{   form_init_elements()

    /**
     *  This method initializes the form elements that will be used.  Please refer
     *  to {@link SearchForm} for more information.
     */
    function form_init_elements()
    {
        $search     = new FEText(_("Search"));
        //TODO: Fix this
        $type       = new FERadioGroup(_("Type"), $this->_get_types());

        /* Vertical buttons */
        $type->set_br_flag(true);
       
        /* Don't show required field marker */
        $this->set_required_marker(" ");
        
        /* Get an array in the form [module][object][display name] */
        foreach (Module::do_action('search') as $var=>$val) {
            $module = $val['module'];
            $mod_obj = new $module();
            $objects = $mod_obj->searchGetObjects();
            $this->object_data[$module] = $objects;
        }

        

        /*Turn off validation of the "Field" listbox and "Types" Radio Group*/

        $this->add_element($search);
        $this->add_element($type);
        
        $this->set_action($_SERVER['SCRIPT_NAME']);
    }

//}}}
//{{{   form_init_data()
    /*
     *  This method initializes form elements. In this case
     *  the "Object" master listbox is set to the first object
     *  and the slave list is filtered down. The radio group
     *  is also set accordingly.
     *
     */
    function form_init_data()
    {
        $type = &$this->get_element("Type");
        $type->set_value(key($type->_data_list[0]));
    }
//}}}
//{{{  form()
    function form()
    {
        $div = html_div("form");
        $table = html_table("45%");

        $div->add($this->_search_javascript());

        /* Add Search field */
        $tr = html_tr("odd");
        $tr->add($this->element_label(_("Search")), $this->element_form(_("Search")));        
        $table->add($tr);

        /* Cosmetics */
        $table->add_row("&nbsp;");

        $tr = html_tr("even");
        $fields = html_span("formlabel");
        $fields->add("&nbsp;", "Objects");
        $tr->add($fields);
        $table->add($tr); 

        /* Build the object's checkbox list */
        $i = 0; //for two rows of object cbs
        $tr = html_tr("even");
        $tr->add("&nbsp;");
        foreach ($this->object_data as $name=>$module) {
            foreach ($module as $obj_type=>$obj_name) {
                $i++;
                $type = ($i & 2 ? "even" : "odd");
                if ($i & 1)  {
                    $tr = html_tr($type);
                    $tr->add("&nbsp;");
                }
                $cb = "<input type='checkbox' onChange='objClicked(\"$name::$obj_type\", this.checked)' name='data[$name][$obj_type]'>$obj_name</input>";
                //$tr->add($cb);
                $mod = new $name();

                /* Create an area (div) for the field checkboxes to live in */
                //TODO: Fix this string prob. (Pass by ref?)
                $sub_cb = "";
                foreach ($mod->searchGetFields($obj_type) as $field_name=>$field)   {
                    $sub_cat = html_fieldset();
                    $sub_cat->set_id("$name::$obj_type");
                    $sub_cat->set_style("visibility: hidden; display: none;");
                    $sub_cb.= "<input type='checkbox' value='$field_name' name='data[$name][$obj_type][$field]'>$field_name<br></input>";
                }
                $sub_cat->add($sub_cb);
                $tr->add(container($cb, $sub_cat));
                if (!($i & 1))
                    $table->add($tr);
            }
        }

        $table->add_row("&nbsp;");
        
        /* Add the "type" checkbox */

        $tr = html_tr("even");
        $obj = $this->element_form(_("Type"));
        $label = $this->element_label(_("Type"));
        $tr->add(container("&nbsp;", $label), $obj);
        
        $table->add($tr);
        $table->add_row("&nbsp;");

        /* Set the action of the form to this script */
        $this->set_cancel_action($_SERVER["SCRIPT_NAME"]);

        /*  Add submit and cancel buttons   */
        $tr = html_tr("odd");

        /* Add the form action and cancel button */
        /* Because we are posting back to filter the listboxen, etc.. */
        /* We need to change the from action when submitting */
        $tr->add("&nbsp;", $this->add_action(_("Search")/*, false, "this.form.searchform_action.value='submit'"*/), $this->add_cancel());
        $table->add($tr);
        
        /* Add the hidden elements */
        $div->add($this->form_hidden());

        /* Add the table to the form div */
        $div->add($table);


        return $div;
    }
//}}}
    
//{{{   form_hidden()

    /**
     *  This method establishes the hidden elements for the form.
     *
     *  @return Container   Container with the hidden elements for the form
     */

    function form_hidden()
    {
        $action =  new FEHidden(_("action"), "search");
        $this->add_element($action);

        return $this->element_form(_('action'));                                
    }

//}}}
//{{{   form_action()

    /**
     *  This method handles the search action. It may be
     *  filtering down the slave drop down list or 
     *  handling search requests
     *
     *  @return Boolean True if action succeeds
     */
    function form_action()
    {
        $res = true;
        switch (strtolower($this->get_action()))
        {

        case _("search"):
            break;

        default:
            /* Re-populate the "Field" listbox and the "Type" radio button group */
            break;
        }
        return $res;
    }

//}}}
//{{{   form_success()

    /**
     * 
     *  
     *
     *  @return DIVTag  $div    Div that contains the success message
     */
    function form_success() 
    {
    }
//}}}

// {{{ _get_types()
    /**
      * Get valid search types from Search module, like exact, fuzzy etc.
      *
      * @private
      * @return Array valid search types
      */
    function _get_types()
    {
        $result = array();
        $m = new maintain_search();
        $ary = $m->searchGetAcceptableTypes("host"); 
        foreach ($ary as $var=>$val)
            $result[$val] = $val;
        return $result;
        
    }
// }}}
// {{{ _search_javascript()
    /**
      * Return the javascript block so that it can be included into the 
      * search page header
      *
      * @private
      * @return String Javascript block for the search page
      */
    function _search_javascript()
    {
        $js = <<<EOT
<script language='Javascript'>
function objClicked(id, checked) {
    e = document.getElementById(id);
    if (checked)    {
        e.style.visibility = 'visible';
        e.style.display = 'block';
    }
    else    {
        e.style.visibility = 'hidden';
        e.style.display = 'none';
    }
     
    cbs = e.getElementsByTagName('input');
    for (i = 0; i < cbs.length; i++)
        cbs[i].checked = false;
}
</script>
EOT;
        return $js;
    }
// }}}
}

?>
