<?php

/* vim: set expandtab tabstop=4 shiftwidth=4 foldmethod=marker: */
// $HeadURL: https://svn.osuosl.org/public/maintain-project/branches/maintain-3/modules/module.php $
// +------------------------------------------------------------------------+
// | Maintain - DNS/DHCP Managment Tool                                     |
// +------------------------------------------------------------------------+
// | Copyright (c) 2004 Open Source University                              |
// +------------------------------------------------------------------------+
// | This program is free software; you can redistribute it and/or          |
// | modify it under the terms of the GNU General Public License            |
// | as published by the Free Software Foundation; either version 2         |
// | of the License, or (at your option) any later version.                 |
// |                                                                        |
// | This program is distributed in the hope that it will be useful,        |
// | but WITHOUT ANY WARRANTY; without even the implied warranty of         |
// | MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the          |
// | GNU General Public License for more details.                           |
// |                                                                        |
// | You should have received a copy of the GNU General Public License      |
// | along with this program; if not, write to the Free Software            |
// | Foundation, Inc., 59 Temple Place - Suite 330,                         |
// | Boston, MA  02111-1307, USA.                                           |
// +------------------------------------------------------------------------+
// | Author(s): Daniel Robert <robert@osuosl.org>                           |
// |            Frederic Wenzel <fred@osuosl.org>                           |
// +------------------------------------------------------------------------+
// $Id: module.php 4779 2006-08-24 22:37:07Z wenzel $

/**
  * module.php - Contains the {@link Module} class as well
  * as other base classes for module types (i.e. Auth)
  *
  * @author $Author: wenzel $
  * @filesource
  * @package    modules
  */

/** Used in a helper method for testing enabled status */
define("ENABLE", "1");

/** Used in a helper method for testing enabled status */
define("DISABLE", "0");

/**
  * A maintain module
  *
  * @package    modules
  */
class Module {

// {{{ vars

    /**
      * The configuration array (read from the config file) for this module
      */
    var $config;

    /**
      * The config file name (or false if none)
      */
    var $config_filename = false;

    /**
      * Has the module config errors? Is set during automatic check process
      * for other classes to find out afterwards
      */
    var $hasConfigErrors = false;

    /**
      * Path to this module
      * @static
      * @access private
      */
    var $_path;

    /**
     * Force backend action?
     */
    var $force_backend = false;

// }}}

/* Constructor */
// {{{ constructor

    /**
      * Constructor
      *
      * calls initialize()
      */
    function Module()
    {
        // set the path (i.e. /modules/<name>)
        $this->_path = MODDIR.'/'.$this->getName();

        // get the config, if there is any
        if ($this->needsConfigFile()) {
            $this->config_filename = $this->_path . '/' . $this->getName() . '.cfg';

            // load config file if it's valid
            $validation = $this->validateConfigFile();
            if ($validation !== true) { // validation failed
                $this->_configError($validation);
            } else {
                $this->config = parse_ini_file($this->config_filename, true);
            }
        }

        // do anything else
        $this->initialize();
    }

// }}}

/* Getters */
// {{{ getName()

    /**
      * Get the name of this module
      *
      * @final
      * @return string  the classname of this module's base class (lowercased)
      */
    function getName()
    {
        return get_class($this);
    }

// }}}
// {{{ getConfig()

    /**
      * Return the config array for this module
      *
      * @final
      * @return array   reference to the config array for this module
      */
    function getConfig()
    {
        return $this->config;
    }

// }}}

/* Methods that should be overwritten */
// {{{ initialize()

    /**
      * Parses and stores this module's config internally,
      * and sets the path for convenience
      *
      * Called from the constructor
      *
      * Throws a PEAR_Error on failure
      *
      * @access protected
      */
    function initialize() 
    {
        return;
    }

// }}}
// {{{ description()

    /**
      * Provide a description of this module.  This should include 
      * the basic idea, some information on how it works, and what is affected
      * by its install and uninstall processes
      *
      * @return string  description of this module
      */
    function description()
    {
        return _("No description provided for this module");
    }

// }}}
// {{{ register_hooks() (register methods to Maintain module hooks)

/**
  * Connect this module to the module hooks it should reply to
  * Should be overridden by every module, and contain a couple of
  * add_action(.) and add_filter(.) statements
  *
  * @access public
  * @abstract
  * @return void
  */
    function register_hooks()
    {
        // Module::add_action(hook, method, priority)
        // Module::add_filter(hook, method, priority)
        return;
    }
// }}}

/* Search-related methods */
// {{{ searchGetObjects()

    /**
      * This function will return an array of Strings mappings 
      * (internal representations to human-readable object names)
      * that the module will support search for.
      * example: array(
      *   'ip' => 'IP Address'
      * )
      * False otherwise.
      *
      * @access public
      * @return mixed array of object identifiers, or false.
      */
    function searchGetObjects()
    {
        return false;
    }
    
// }}}
// {{{ searchGetFields(.)

    /**
      * given an object name (as defined by searchGetObjects()), this function
      * returns an array of fields that are searchable through the search form
      * for the given object.
      * Format: associative array of String mappings internal => human-readable
      * Example: array(
      *   'fqdn' => 'Fully qualified domain name'
      * )
      *
      * @param object name
      * @return mixed array of searchable fields or false
      */
    function searchGetFields($objName)
    {
        return false;
    }
      
// }}}
// {{{ searchGetAcceptableTypes(.)

    /**
      * given an object name (as defined by searchGetObjects()),
      * returns an array of acceptable search types for these.
      * (e.g. Exact, Fuzzy, Regex...)
      * false otherwise.
      *
      * @param object name
      * @return mixed array of search types or false
      */
    function searchGetAcceptableTypes($objName)
    {
        return false;
    }
      
// }}}
// {{{ searchDisplayColumns()

    /**
      * Will return an array of the form (Display => backend) that will
      * be used for the result data list. "display" is the name displayed
      * to the user, backend will be used internally.
      * false otherwise.
      *
      * @param object that we are searching
      * @return mixed array of display columns or false
      */
    function searchDisplayColumns($object)
    {
        return false;
    }
      
// }}}
// {{{ search(.)

    /**
      * executes an actual search on the objects searchable by this
      * module.
      * false otherwise.
      *
      * @param search query string
      * @param object name of object type to search for
      * @param attribute field of objects to search in
      * @param type of search to be executed as defined by searchGetAcceptableTypes
      * @return mixed array of display columns or false
      */
    function search($query, $objname, $field, $type)
    {
        return false;
    }
      
// }}}
// {{{ searchFetch(.)

    /**
      * Returns a field value for a given object and UID (false otherwise).
      * The information will be used to populate the rows of the search
      * result table
      *
      * @param object name of object type to search for
      * @param unique identifier of object to be queried
      * @param field name of attribute to be returned
      * @return string representation of the queried object attribute,
      * false otherwise.
      */
    function searchFetch($objname, $uid, $field)
    {
        return false;
    }
      
// }}}

/* Abstract Methods */
// {{{ _install()

    /**
      * Perform any actions that need doing before this module
      * is usable.
      *
      * Do not modify the modules table, as this will be done
      * by the {@link Module} class in it's {@link install} method
      *
      * @abstract
      * @access protected
      * @return mixed   true on success or PEAR_Error on failure
      */
    function _install() {}

// }}}
// {{{ _uninstall()

    /**
      * Perform any actions that need doing to remove this module from Maintain
      * Typically, this involves removing relevant database tables
      *
      * Do not modify the modules table, however, this will be done
      * by the {@link Module} class in it's {@link uninstall} method
      *
      * @abstract
      * @access protected
      * @return mixed  true on success, PEAR_Error on failure
      */
    function _uninstall() {}

// }}}

/* Final Methods */
// {{{ isEnabled()

    /**
      * Check to see if this module has been enabled by the administrator
      * The name MUST be set before this method will work
      *
      * @final
      * @return boolean true if enabled, false if not
      */
    function isEnabled()
    {
        // get the enabled column from this module's row in the db
        $mod = DB_DataObject::Factory('modules');
        $mod->selectAdd();
        $mod->selectAdd('enabled');

        // if the module exists, return the enabled value as a boolean
        if ($mod->get($this->getName())) {
            return ($mod->enabled) ? true : false;
        }
       
        // if not, return false
        return false;
    }

// }}}
// {{{ setEnabled()

    /**
      * Enable this module
      *
      * @final
      * @param  mixed   true/false, 'true'/'false', 1/0
      * @return boolean true on successful set
      */
    function setEnabled($en)
    {
        $dbdo = DB_DataObject::Factory('modules');
        $dbdo->name = $this->getName();
        
        if ($en === true || $en === 'true' || $en === 1) {
            $dbdo->enabled = 1;
        } elseif ($en === false || $en === 'false' || $en === 0) {
            $dbdo->enabled = 0;
        } else {
            return false;
        }

        // is_numeric is used because if the module is already enabled,
        // and we set it to enabled, it will return '0', as there was no update
        // but the operation is still considered successful
        return (is_numeric($dbdo->update())) ? true : false;
    }

// }}}

/* Static Methods */
// {{{ install()

    /**
      * Installs a module by name.  Function behaves the same
      * if the module already existed.
      *
      * This method accomplishes two major tasks:
      * 1) Add the class to the Modules table (as disabled)
      * 2) Call the module's internal install method
      *
      * @static
      * @final
      * @param  string  name (classname) of the module to install
      * @return mixed   true if module installs or already installed, 
      *                 PEAR_Error on failure
      */
    function install($mod, $force = false)
    {
        /*
         * 1) If module doesn't exist, complain
         *    Same for Core modules, which may never be additionally installed.
         * 2) If everything's alright, install it as DISABLED
         * 3) call the child class' _install() method
         */

        // make sure a name was given
        if (!$mod) {
            return new PEAR_Error(_("No module name was given for installation"));
        }

        /* Modules that aren't installed have not been included yet */
        include_once MODDIR . "/$mod/$mod.php";

        /* Check if module is a core module. Deny installation in that 
           case, since no module not shipped with Maintain can be core */
        if (!$force) {
            if (get_parent_class($mod) == 'core_module') return new PEAR_Error(_('This module is a core module. '
                .'By convention, core modules are part of the Maintain distribution package '
                .'and can not be additionally installed.'));
        }
        
        /* Test if module exists and add to table if not */
        $dbdo = DB_DataObject::Factory('modules');
        $dbdo->name = $mod;
        if (!$dbdo->find()) {

            $dbdo->enabled = 0;

            // insert it or exit in disgrace
            if (!$dbdo->insert()) {
                return new PEAR_Error(_("Could not add module to Modules table"));
            }
        } // module exists?

        /* call the child class' _install() method */
        if (!class_exists($mod)) {
            $dbdo->delete();
            return new PEAR_Error(_("Could not find module") . " $mod");
        }
        $module = new $mod;
        $result = $module->_install();

        // if we couldn't install the module, 
        // remove it from the modules table
        // and propogate the error
        if (PEAR::isError($result) || !$result) {
            $dbdo->delete();
            return $result;
        }

        // if module was successfully installed, propagate needsConfigFile
        // to DB for the automatic config check to work properly
        $dbdo->needsConfigFile = $module->needsConfigFile();
        $dbdo->update();

        return true;

    }

// }}}
// {{{ uninstall()

    /**
      * Remove a module and call its internal uninstall method
      *
      * This method accomplishes two main tasks:
      * 1) Call the module's internal uninstall method
      * 2) Remove the class from the modules table
      *
      * @static
      * @final
      * @param  string  name (classname) of module to uninstall
      * @return mixed   true if module is removed from modules table
      *                 and module's internal uninstall runs successfully,
      *                 PEAR_Error otherwise
      */
    function uninstall($mod, $force = false)
    {
        // make sure a module name was given
        if (!$mod) {
            return false;
        }

        /* Check if module is a core module. Deny installation in that 
           case, since no module not shipped with Maintain can be core */
        if (!$force) {
            if (get_parent_class($mod) == 'core_module') return new PEAR_Error(_('This module is a core module. '
                .'By convention, core modules are part of the Maintain distribution package '
                .'and can not be uninstalled.'));
        }
        
        // remove the module from the modules table
        $dbdo = DB_DataObject::Factory('modules');
        $dbdo->name = $mod;
        $dbdo->delete();

        // call the module's internal uninstall method
        if (class_exists($mod)) {
            $module = new $mod();
            $success = $module->_uninstall();

            // propogate errors
            if (PEAR::isError($success) || !$success) {
                return $success;
            }
        }

        return true;

    }

// }}}
// {{{ upgrade()

    /**
      * Run a module's upgrade script
      *
      * @static
      * @final
      * @param  string  name (classname) of module to upgrade
      * @return mixed   true if module is successfully upgraded
      *                 or was already upgraded,
      *                 PEAR_Error otherwise
      */
    function upgrade($mod)
    {
        // make sure a module name was given
        if (!$mod) {
            return false;
        }

        // call the module's internal upgrade method
        if (class_exists($mod)) {
            $module = new $mod();

            // if there's no upgrade function, we're ok
            if (!method_exists($module, '_upgrade')) {
                return true;
            }

            $success = $module->_upgrade();

            // propogate errors
            if (PEAR::isError($success) || !$success) {
                return $success;
            }
        }

        return true;

    }

// }}}
// {{{ disable()

    /**
      * Disable a module by name
      *
      * @static
      * @final
      * @param  string  name of module to disable
      * @return boolean true if module is disabled (or was already)
      */
    function disable($mod, $force = false)
    {
        return Module::_enable_disable($mod, DISABLE, $force);
    }

    // }}}
// {{{ enable()

    /**
      * Enable a module by name
      *
      * @static
      * @final
      * @param  string  name of module to enable
      * @return boolean true if module is enabled (or was already)
      */
    function enable($mod, $force = false)
    {
        return Module::_enable_disable($mod, ENABLE, $force);
    }

    // }}}
// {{{ getDashboardItems()

	/**
	  * There is a dashboard widget specifically for widget links.
	  * This method queries each enabled module for their dashboard info
	  *
	  * This method MUST be invoked from an htdocs page, so that all the
	  * appropriate phpHtmlLib api is available
	  * 
	  * @link	http://phphtmllib.newsblob.com/javadoc.php phpHtmlLib API Documentation
	  * @static
	  * @final
      * @param  string      relevant dashboard:  'main' or 'admin'
	  * @return	Container	a phpHtmlLib container object to be added to the
							dashboard's modules widget
	  */
	function getDashboardItems($type = 'main')
	{
		// create the container
		$cont = container();

        $results = Module::do_action("module_dashboard_{$type}");
        $anyoutput = false;
        foreach ($results as $result) {
            if ($result['result']) {
                // yes, we have module output
                $anyoutput = true;

				// get the rest of the dashboard info from the module
				$cont->add($result['result'], html_br());
            }
        }

        if (!$anyoutput) {
			$cont->add(_("No module links available"));
        }
        
        return $cont;
	}

// }}}
// {{{ getAdminDashboardItems()

    /**
      * Calls {@link getDashboardItems} with type = 'admin'
      *
 	  * @link	http://phphtmllib.newsblob.com/javadoc.php phpHtmlLib API Documentation
	  * @static
	  * @final
	  * @return	Container	a phpHtmlLib container object to be added to the
							dashboard's modules widget
	  */
    function getAdminDashboardItems()
    {
        return getDashboardItems('admin');
    }

// }}}
/* Static Module API Methods */
// {{{ add_action(.)

    /**
      * Connect a module's method to a named module hook of type action.
      *
      * @access public
      * @static
      * @param String $hook identifying string of the named hook, from module hook documentation
      * @param mixed $method either name of the module's method to be connected
      *     to the hook, or an array('modulename', 'methodname')
      * @param int priority optional priority of this method, between 1 and 20. If non is
      *     given, default 10 is assumed.
      */
    function add_action($hook, $method, $priority = 10, $access = GUEST)
    {
        Module::_add_action_or_filter('action', $hook, $method, $priority, $access);
    }
// }}}
// {{{ add_filter(.)

    /**
      * Connect a module's method to a named module hook of type filter.
      * Same parameters as {@link add_action}
      *
      * @access public
      * @static
      */
    function add_filter($hook, $method, $priority = 10, $access = GUEST)
    {
        Module::_add_action_or_filter('filter', $hook, $method, $priority, $access);
    }
// }}}
// {{{ do_action(.)
    /**
      * Execute an 'action' module hook.
      * Takes a dynamic amount of arguments, according to the module hook
      * documentation, usually object IDs etc.
      *
      * @access public
      * @static
      * @param      string  $hook   action hook name to be executed
      * @global     array   $module_filters modules connected to module hooks
      */
    function do_action($hook)
    {
        global $module_filters;

        
        $hook = strtolower($hook); // case insensitive module hooks
        if (!isset($module_filters['action'][$hook])) return array(); // nothing to do: empty result array
        
        $numargs = func_num_args();
        if ($numargs == 0) {
            return false; // hook name required
        } elseif ($numargs == 1) {
            $args = array();
        } else {
            $args = array_slice(func_get_args(), 1);
        }

        // sort by priority, then execute
        ksort($module_filters['action'][$hook]);

        // make result array
        $results = array();

        foreach ($module_filters['action'][$hook] as $priority => $functions) {
            if (!is_null($functions)) {
                foreach ($functions as $function) {
                    if(!Module::_check_access($function['access'])) continue;
                    if (!class_exists($function['module'])) continue; // unknown module
                    $mod = new $function['module'];
                    if (!$mod->isEnabled()) continue; // disallow disabled modules
                    
                    if (!is_callable(array($mod, $function['method']))) continue; // unknown method
                    
                    // call the module's method that was connected to this hook, passing the argument array
                    $retval = call_user_func_array(array(&$mod, $function['method']), $args);

                    $results[] = array('module' => $function['module'],
                                       'method' => $function['method'],
                                       'result' => $retval);
                }
            }
        }

        return $results;
    }
// }}}
// {{{ apply_filter(.)

    /**
      * Execute a module filter hook on a given string
      * In addition to the string to be filtered, it takes a dynamic 
      * amount of arguments, according to the module hook documentation,
      * usually object IDs etc.
      *
      * @access public
      * @static
      * @param      string  $hook   filter hook name to be executed
      * @param      mixed   $apply_on string/object/whatever to be filtered/modified
      * @global     array   $module_filters modules connected to module hooks
      */
    function apply_filter($hook, $apply_on)
    {
        global $module_filters;
        $hook = strtolower($hook); // case insensitive module hooks
        if (!isset($module_filters['filter'][$hook])) return $apply_on; // nothing to do
        
        $numargs = func_num_args();
        if ($numargs < 2) {
            return false; // hook name required
        } elseif ($numargs == 2) {
            $otherargs = array();
        } else {
            $otherargs = array_slice(func_get_args(), 2);
        }

        // sort by priority, then execute
        ksort($module_filters['filter'][$hook]);
        // initial filter result is the given argument
        $filtered = $apply_on;

        foreach ($module_filters['filter'][$hook] as $priority => $functions) {
            if (!is_null($functions)) {
                foreach ($functions as $function) {
                    if(!Module::_check_access($function['access'])) continue;
                    if (!class_exists($function['module'])) continue; // unknown module
                    $mod = new $function['module'];
                    if (!$mod->isEnabled()) continue; // disallow disabled modules
                    if (!is_callable(array($mod, $function['method']))) continue; // unknown method

                    // make argument array out of current filtered string and other args
                    $args = array_merge(array($filtered), $otherargs);
                    
                    // call the module's method that was connected to this hook, passing the argument array
                    $filtered = call_user_func_array(array(&$mod, $function['method']), $args);
                }
            }
        }

        return $filtered;
    }
// }}}
// {{{ do_single_action(.)
    /**
      * Execute an 'action' module hook on one single module.
      * Differs from do_action only in the fact that the hook is not
      * executed on every module that's registered to the named hook
      * but only to a single module.
      *
      * @access public
      * @static
      * @param      string  $module name of the module to execute the action on
      * @param      string  $hook   action hook name to be executed
      * @return     result from action call, false if none was executed
      * @global     array   $module_filters modules connected to module hooks
      */
    function do_single_action($module, $hook)
    {
        global $module_filters;
        $hook = strtolower($hook); // case insensitive module hooks
        if (!isset($module_filters['action'][$hook])) return false; // nothing to do: empty result array
        if (!class_exists($module)) return false;   // no such module?
        
        $numargs = func_num_args();
        if ($numargs <= 1) {
            return false;
        } elseif ($numargs == 2) {
            $args = array();
        } else {
            $args = array_slice(func_get_args(), 2);
        }

        // sort by priority
        ksort($module_filters['action'][$hook]);
        
        // go through the modules registered to this hook
        // and try to find "our" module. If you do, execute the function.
        // If not, cry.
        foreach ($module_filters['action'][$hook] as $priority => $functions) {
            if (!is_null($functions)) {
                foreach ($functions as $function) {
                    if(!Module::_check_access($function['access'])) continue;
                    if ($function['module'] != $module) continue; // not the chosen module
                    $mod = new $function['module'];
                    if (!$mod->isEnabled()) return false; // disallow disabled modules
                    
                    if (!is_callable(array($mod, $function['method']))) return false; // unknown method
                    
                    // call the module's method that was connected to this hook, passing the argument array
                    $retval = call_user_func_array(array(&$mod, $function['method']), $args);
                    
                    return $retval;
                }
            }
        }

        // if something was found and executed, we never get here.
        return false;
    }
// }}}
// {{{ apply_single_filter(.)

    /**
      * Execute a module filter hook on a given string
      * Differs from apply_filter in that it does only apply the filter
      * to one single module chosen in the parameters.
      *
      * @access public
      * @static
      * @param      string  $module name of the module whose filter to apply
      * @param      string  $hook   filter hook name to be executed
      * @param      mixed   $apply_on string/object/whatever to be filtered/modified
      * @global     array   $module_filters modules connected to module hooks
      */
    function apply_single_filter($module, $hook, $apply_on)
    {
        global $module_filters;
        $hook = strtolower($hook); // case insensitive module hooks
        if (!isset($module_filters['filter'][$hook])) return $apply_on; // nothing to do
        if (!class_exists($module)) return $apply_on; // unknown module
        
        $numargs = func_num_args();
        if ($numargs < 3) {
            return false;
        } elseif ($numargs == 3) {
            $otherargs = array();
        } else {
            $otherargs = array_slice(func_get_args(), 3);
        }

        // sort by priority, then execute
        ksort($module_filters['filter'][$hook]);
        // initial filter result is the given argument
        $filtered = $apply_on;

        foreach ($module_filters['filter'][$hook] as $priority => $functions) {
            if (!is_null($functions)) {
                foreach ($functions as $function) {
                    if(!Module::_check_access($function['access'])) continue;
                    if ($function['module'] != $module) continue; // not "our" module
                    
                    $mod = new $function['module'];
                    if (!$mod->isEnabled()) return $apply_on; // disallow disabled modules
                    if (!is_callable(array($mod, $function['method']))) return $apply_on; // unknown method

                    // make argument array out of current filtered string and other args
                    $args = array_merge(array($filtered), $otherargs);
                    
                    // call the module's method that was connected to this hook, passing the argument array
                    $filtered = call_user_func_array(array(&$mod, $function['method']), $args);
                    return $filtered; // one single filter could be applied. All we want, let's go.
                }
            }
        }
        
        // if a filter could be applied, we never get here.
        return $apply_on;
    }
// }}}

/* Methods for Config File Intergrity Checking */
// {{{ needsConfigFile()

    /**
      * Returns, if the module needs a config file at all. This will be used
      * by the automated config file integrity checks, for example.
      * Defaults to false.
      * Can be called statically.
      *
      * @return boolean .cfg file vital for the module to work?
      */
    function needsConfigFile()
    {
        return false;
    }
// }}}
// {{{ validateConfigFile()

    /**
      * If the module needs a config file, this checks its validity.
      * The function defaults to a simple check: does parsing with
      * PHP's parse_ini_file work? For a more sophisticated validity
      * check, module writers would want to overwrite this method,
      * call it by super::validateConfigFile() and extend it.
      * Will return true if no config file is needed.
      *
      * @param force check? Otherwise true is returned if conf was not
      *     changed since the last check
      * @return true .cfg file valid or PEAR_Error
      */
    function validateConfigFile($force = false)
    {
        // no config file is trivially valid
        if (!$this->needsConfigFile()) return true;
        
        // is the config file there and readable?
        if (!is_readable($this->config_filename)) 
            return new PEAR_Error(_('Config file is either not present or not readable!'));
        
        // find the module record in the DB
        $mods = DB_DataObject::Factory('modules');
        $mods->name = get_class($this);
        $mods->find();
        if (!$mods->fetch()) {
            $mods->free();
            return new PEAR_Error(_("Module not found in database: it is not installed correctly!"));
        }
        // if the check is not forced, ask the DB if the config
        // file was changed since last check
        if (!$force) {
            if ($mods->config_mtime == filemtime($this->config_filename))
                return true;
        }
        
        // try to parse the file without errors
        $parse_test = @parse_ini_file($this->config_filename, true) or $parse_test = null;
        if (is_array($parse_test)) {
            // write config file modification time to database
            $mods->config_mtime = filemtime($this->config_filename);
            $mods->update();
        } else {
            return new PEAR_Error(_('Parse error in config file! Please check its syntax!'));
        }

        return true;
        
    }
// }}}

/* Helper Methods (private) */
// {{{ _enable_disable()

    /**
      * Helper function to enable/disable a module by name
      *
      * @static
      * @final
      * @access private
      * @param  string  name of module to enable
      * @param  int     0 for disable, 1 for enable
      * @return boolean true if module is enabled (or was already)
      */
    function _enable_disable($mod, $en, $force)
    {
        $module = DB_DataObject::Factory('modules');

        // get the module from the DB or fail
        if (!$module->get($mod)) {
            return false;
        }

        /* Check if module is a core module. Deny installation in that 
           case, since no module not shipped with Maintain can be core */
        if (!$force) {
            if (get_parent_class($mod) == 'core_module') return new PEAR_Error(_('This module is a core module. '
                .'By convention, core modules are part of the Maintain distribution package '
                .'and can not be enabled or disabled.'));
        }
        
        $module->enabled = $en;

        return is_numeric($module->update());
    }

// }}}
// {{{ _configError()

    /**
      * config Error. Complain and disable module!
      *
      * @final
      * @access private
      * @param  mixed   PEAR_Error to be displayed or nothing
      */
    function _configError($error = null)
    {
        // complain to admin
        $errormsg = $error->getMessage();
        $errormsg .= "\n\n". _("The module will be disabled automatically. To re-enable it, "
            .'fix the config file first, then go to the bin directory and execute ')
            .'"./modules.php -e '.$this->getname().'".';

        $error = new PEAR_Error($errormsg);
        
        Email::adminEmail($error, $this->getName());
        
        $this->setEnabled(false);
        $this->hasConfigErrors = true;
    }

// }}}
// {{{ _add_action_or_filter(.)

    /**
      * Add a method to either an action or filter hook
      * is called by {@link add_action} or {@link add_filter} respectively
      *
      * The two arrays module_actions and module_filters are initialized
      * in modules/include.php, so that they are globally available
      * all over Maintain
      *
      * @access private
      * @static
      * @global array   $module_filters array containing modules tied to action/filter hooks
      * @param  string  hook type: 'action' or 'filter'
      * @param  string  hook name (according to documentation)
      * @param  string  function name to connect to the hook (current module) or
      *     array('module name', 'method name')
      * @param  int priority, between 1 and 20
      * @param  int minimum access level needed to execute this module function
      */
    function _add_action_or_filter($hooktype, $hook, $method, $priority, $access)
    {
        global $module_filters;

        if ($hooktype != 'action' && $hooktype != 'filter') return false;   // illegal hook type

        $hook = strtolower($hook);  // hook names are case-insensitive

        if (is_array($method)) {    // no need to guess calling module name
            $modulename = $method[0];
            $methodname = $method[1];
            
        } elseif (is_string($method)) {
            // TODO if there is any cleaner way to trace back to the
            // class in which add_action was called, do it here.
            $backtrace = debug_backtrace();
            $modulename = $backtrace[3]['args'][0][0];
            $methodname = $method;
            
        } else { // invalid
            return false;
        }
        
        // core modules can override priority ranges
        // all others must obey them or they get the default value of 10
        $isCoreModule = (strtolower(get_parent_class($modulename)) == 'core_module');
        if (!is_int($priority) || (!$isCoreModule && ($priority < 1 || $priority > 20)))
            $priority = 10;

        /* add action or filter to filter array */
        // check that we don't already have the same filter at the same priority
        if (isset($module_filters[$hooktype][$hook]["$priority"])) {
            foreach($module_filters[$hooktype][$hook]["$priority"] as $filter) {
                if ($filter['module'] == $modulename && $filter['method'] == $methodname) {
                    return true;
                }
            }
        }
        
        // So the format is module_filters['hooktype']['hookname']['array of priorities']['array of ['array (module, method, access level)]']
        $module_filters[$hooktype][$hook]["$priority"][] = array('module' => $modulename, 'method' => $methodname, 'access' => $access);
        return true;
                                                                                                                
    }
// }}}

// {{{ _check_access()
/**
  * Checks the access level of the user
  * @param $access access level
  * @return bool
  */
function _check_access($access)
{
    global $user;
    
    // if we don't have a user object yet (i.e. before logging in),
    // only allow access to modules that do not require a user level
    if (!is_object($user)) return ($access <= NO_ACCESS);
    
    if ($user->get_access_level(null) < $access)
        return false;

    return true;
}
//}}}

} //module class


/**
  * Maintain module that implements user authentication
  *
  * @package    modules
  */
class AuthModule extends Module
{

// {{{ vars

    /**
      * Status (message) relevant to last call to $this->auth().
      * @see AuthModule::getStatus()
      */
    var $_status;

// }}}
    function register_hooks()
    {
        Module::add_action('auth_modules', 'get_name');
    }

    function get_name()
    {
        return get_class($this);
    }

// {{{ auth()

    /**
      * Authenticate a user by name/password
      *
      * The internal '_status' variable should be set as a result
      * of this function.  Values are something descriptive, such as
      * 'success', 'bad password', etc.
      *
      * @param  string  username of user to authenticate
      * @param  string  password of user
      * @return boolean true on successful authentication
      */
    function auth($username, $password)
    {
        $this->setStatus = _("Auth function not implemented");
        return false;
    }

// }}}
// {{{ getStatus()

    /**
      * See the status of the last authentication attempt
      *
      * @access public
      * @final
      * @return string  message relating to last call to {@link auth}
      */
    function getStatus()
    {
        return $this->_status;
    }

// }}}
// {{{ hasUser()

    /**
      * Determine if this authentication server contains a user of given username
      *
      * @param  string  username of user to query for
      * @return boolean true if username found
      */
    function hasUser()
    {
        return true;
    }

// }}}

} //AuthModule class

?>
